
function [FPrate AvgNused Capped]=...
    simulateExperiments(policy,M,alpha,w,Ninit,Nincr,Nmax);
%  [FPrate AvgNused LimitedByNmax]=...
%     simulateExperiments(policy,M,alpha,w,Ninit,Nincr,Nmax);
%
% Estimates by simulation the false positive rate caused by uncorrected
% "N-hacking", see Reinagel, P "Is N-Hacking Ever OK?" bioRxiv 2019
%
% INPUTS
% policy    string, either 'Asymmetric' or 'Symmetric' 
%           note that a fixed-N policy is simply w=0 on either policy
% M         the number of experiments to simulate, e.g. 10^6
% alpha   	significance criterion, e.g., 0.05
% w         closeness criterion: how close to alpha must a P value be
%           to add samples after testing
% Ninit     initial sample size, e.g. 12 (must be >1)
% Nincr     number of samples to add if P is close to alpha, e.g., 1
% Nmax      maximum number of total samples before quitting
%
% OUTPUTS
% FPrate   	fraction of experiments with H=1 given ground truth H=0
% AvgNused 	on average, number of samples used to reach a conclusion
% Capped    fraction of experiments that terminated sampling due to
%           reaching the limit Nmax

% USAGE AND RIGHTS: Anyone may copy and modify this code, but please retain
% this text in the code or any derivatives and please cite Reinagel (2019)
% "Is N-Hacking Ever OK?" bioRxiv (or the subsequently published version
% listed on bioRxiv) if you use the code for a publication.
% Developed in Matlab 2018a 
% Pamela Reinagel,  2019
 
almost=(1+w)*alpha;%P not significant, but close
just=(1-w)*alpha;%%P significant, but close
H=nan(1,M);
P=nan(1,M);
Nused=nan(1,M);
for expt=1:M,
    %test significance for 2 samples from the same distribution
    A=randn(1,Ninit);
    B=randn(1,Ninit);
    [H(expt) P(expt)]=ttest2(A,B,'alpha',alpha);
    
    if strcmp(policy,'Asymmetric'),%only increment if P close and above
        while P(expt)>=alpha && P(expt)<almost && length(A)<Nmax,
            A=[A randn(1,Nincr)];
            B=[B randn(1,Nincr)];
            [H(expt) P(expt)]=ttest2(A,B,'alpha',alpha);
        end
    elseif strcmp(policy,'Symmetric'), %increment if close below or above
        while (P(expt)>=alpha && P(expt)<almost)  ... %NS but close
                | (P(expt)<alpha && P(expt)>=just)... %signif but close
                && length(A)<Nmax, %stop after reaching max sample
            A=[A randn(1,Nincr)];
            B=[B randn(1,Nincr)];
            [H(expt) P(expt)]=ttest2(A,B,'alpha',alpha);
        end
    else
        error(sprintf('Policy %s not defined',policy));
    end % policy
    Nused(expt)=length(A);
end %loop over the M comparisons
FPrate=sum(H)/M;
AvgNused=nanmean(Nused);
Capped= length(find(Nused>=Nmax))/M;

%for debugging: does N ever exceed Nmax by some amount <Nincr?
if any(Nused>Nmax), 
    fprintf('%.3f of expts had N>Nmax, at most %d more (Nincr=%d)\n',...
        length(tmp)/M, max(Nused)-Nmax, Nincr);
end
end


